/***************************************************************************
 *   Copyright (C) 2003 by Sbastien Laot                                 *
 *   slaout@linux62.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef ITEMDRAG_H
#define ITEMDRAG_H

#include <qstring.h>
#include <qdragobject.h>
#include <qdatastream.h>
#include <qvaluelist.h>
#include <kmultipledrag.h>

class QDataStream;

class Basket;
class Item;
class ItemSelection;

/** Dragging/Copying/Cutting Scenario:
  * - User select some items and cut them;
  * - ItemDrag::toMultipleDrag() is called with a tree of the selected items (see Basket::toSelectionTree()):
  *   - This method create a new KMultipleDrag object, create a stream,
  *   - And then browse all items and call the virtual Item::serialize() with the stream as parameter for them to serialize theire content in the "native format".
  *   - This give the MIME type "application/x-basket-item" that will be used by the application to paste the items exactly as they were.
  *   - Then the method try to set alterante formats for the dragged objects:
  *   - It call successively toText() for each items and stack up the result so theire is ONE big text flavour to add to the KMultipleDrag object
  *   - It do the same with toHtml(), toImage() and toLink() to have those flavours as well, if possible...
  *   - If there is only ONE copied item, addAlternateDragObjects() is called on it, so that Unknown objects can be dragged "as is".
  *   - It's OK for the flavours. The method finally set the drag feedback pixmap by asking every selected items to draw the content to a small pixmap.
  *   - The pixmaps are joined to one big pixmap (but it should not exceed a defined size) and a border is drawn on this image.
  *
  * Pasting/Dropping Scenario:
  *
  * @author Sbastien Laot
  */
class ItemDrag
{
  protected:
	static void serializeItems(     ItemSelection *itemList, QDataStream &stream,         bool cutting );
	static void serializeText(      ItemSelection *itemList, KMultipleDrag *multipleDrag               );
	static void serializeHtml(      ItemSelection *itemList, KMultipleDrag *multipleDrag               );
	static void serializeImage(     ItemSelection *itemList, KMultipleDrag *multipleDrag               );
	static void serializeLinks(     ItemSelection *itemList, KMultipleDrag *multipleDrag, bool cutting );
	static void setFeedbackPixmap(  ItemSelection *itemList, KMultipleDrag *multipleDrag               );
	static Item* decodeHierarchy(QDataStream &stream, Basket *parent, bool moveFiles, bool moveItems, Basket *originalBasket);
  public:
	static QDragObject* dragObject(ItemSelection *itemList, bool cutting, QWidget *source = 0);
	static bool canDecode(QMimeSource *source);
	static Item* decode(QMimeSource *source, Basket *parent, bool moveFiles, bool moveItems);
	static Basket* basketOf(QMimeSource *source);
	static QValueList<Item*> itemsOf(QMimeSource *source);
	static void createAndEmptyCuttingTmpDir();

	static const char *ITEM_MIME_STRING;
};

/** QTextDrag with capabilities to drop GNOME and Mozilla texts
  * as well as UTF-16 texts even if it was supposed to be encoded
  * with local encoding!
  * @author Sbastien Laot
  */
class ExtendedTextDrag : public QTextDrag
{
  Q_OBJECT
  public:
	static bool decode(const QMimeSource *e, QString &str);
	static bool decode(const QMimeSource *e, QString &str, QCString &subtype);
};

#endif // ITEMDRAG_H
